@@ -61,8 +61,10 @@ extern void txt_shutdown(void);
 extern bool txt_is_powercycle_required(void);
 extern void ap_wait(unsigned int cpuid);
 extern int get_evtlog_type(void);
+extern bool is_tpr_supported(void);
 
 extern uint32_t g_using_da;
+extern bool     g_tpr_support;
 #endif      /* __TXT_TXT_H__ */
 
 
